/********************************************************************/
/*    TORQUE (Types Of Relationships QUality Evaluation) toolkit    */
/*                                                                  */
/* Original file name: tviewGraph.h                                 */
/*                                                                  */
/* This software by R.M.                                            */
/********************************************************************/

#ifndef TVIEWGRAPH_H
#define TVIEWGRAPH_H

#define MAX_COMMAND_LENGTH 4096

#include "asgraph.h"
#include "smartstructs.h"
#include "consoletools.h"
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <errno.h>

typedef Graph<ASnode, ASEdgeWeight, ASnode> ConnectedComponent;
typedef ConnectedComponent** ConnCompArray;

/* These are shortcuts to the indexes used to identify each command and/or
   argument */
enum command_index {CMD_EXIT = 0, CMD_QUIT = 1, CMD_DUMP = 2, CMD_SHOW = 3, CMD_HELP = 4, CMD_LOAD = 5,
                    CMD_CLEAR = 6, CMD_ECHO = 7, CMD_SAVE = 8};
enum arg1_index {ARG_GRAPH = 0, ARG_SCC = 1, ARG_SCCS = 2, ARG_CC = 3, ARG_CCS = 4};
enum arg2_index {ARG_INFO = 0, ARG_NODES = 1, ARG_EDGES = 2, ARG_INDEX = 3, ARG_IPCOVERING = 4,
                 ARG_PATHCOVERING = 5, ARG_DIFFERENCES = 6};
enum arg3_index {ARG_ATTENDANCES = 0, ARG_DIRECTED = 1, ARG_UNDIRECTED = 2, ARG_ORIENTED = 3,
                 ARG_UNORIENTED = 4, ARG_CHANGES = 5, ARG_MEANDURATION = 6};

/* This is a list of the actions that should be undertaken on issuing
   a certain command */
enum command_action {ACTION_NONE, ACTION_DUMP, ACTION_SHOW, ACTION_LOAD, ACTION_CLEAR,
                     ACTION_ECHO, ACTION_SAVE, ACTION_QUIT};

// This is a list of possible targets a command can work on
enum command_target {TARGET_GRAPH, TARGET_SCC, TARGET_SCCS, TARGET_CC, TARGET_CCS,
                     TARGET_IPCOVERING, TARGET_PATHCOVERING, TARGET_NODES,
                     TARGET_EDGES, TARGET_INDEX, TARGET_INFO, TARGET_DIFFERENCES,
                     TARGET_NONE, TARGET_ATTENDANCES, TARGET_DIRECTED, TARGET_UNDIRECTED,
                     TARGET_ORIENTED, TARGET_UNORIENTED, TARGET_CHANGES,
                     TARGET_MEANDURATION};

/* The following are strings representing available scripting commands
   and their arguments (of course, not all the combinations are allowed) */
const char commandString[28][13] = {
   "exit", "quit", "dump", "show", "help", "load", "clear", "echo", "save",         // Main commands
   "graph", "scc", "anyscc", "cc", "anycc",                                         // Level 1 arguments
   "information", "nodes", "edges", "index", "ipcovering", "pathcovering", "differences",       // Level 2 arguments
   "attendances", "directed", "undirected", "oriented", "unoriented", "changes", "duration"};   // Level 3 arguments

/* This is an array of indexes that are used to quickly retrieve the
   first string of each argument group */
char argIndex[4] = {0, 9, 14, 21};

// A structure to hold command line parameters
struct cmdLineParameters {
   cmdLineParameters() {
      scriptFile = NULL;
   }
   ~cmdLineParameters() {
      if (scriptFile != NULL) {
         delete [] scriptFile;
      }
   }
   bool force,          /* When this is true, user is not asked for confirmation
                           when overwriting an existing file */
        help;           // Help mode
   char *scriptFile;    /* Name of a file that should be interpreted as a script
                           file (if any) */
};

/* This function prints a given AS graph edge; if the graph it
   comes from is a differential graph, and the second argument
   is true, it prints the directions that edge appeared with
   in the two compared graphs */
void printEdge(ASGraph::graph_edge*, bool);

/* This function prints IP covering information for
   a given edge; `whichGraph' tells the function whether
   to get such information from the first or the second of
   the compared graphs; this is senseless if the current
   one is not a differential graph */
void printIPCovering (ASGraph::graph_edge*, cmp_graph whichGraph = CMPGRAPH_FIRST);

/* This function prints path covering information for
   a given edge; `whichGraph' tells the function whether
   to get such information from the first or the second of
   the compared graphs; this is senseless if the current
   one is not a differential graph */
void printPathCovering (ASGraph::graph_edge*, cmp_graph whichGraph = CMPGRAPH_FIRST);

/* This function performs all the actions that can be requested
   by means of the `dump' command; see its on-line help for a
   more detailed explanation of what such actions are */
void dumpGraph(command_target target[], long int ccNum,
               ASGraph *graph,
               ConnCompArray sccs, long int scclen,
               ConnCompArray ccs, long int cclen);

/* This function prints a distribution that is stored
   inside a BST (each of its nodes keeps count of how
   many items - sccs, ccs, etc. depending on what the
   distribution represents - exist with a number of
   elements - nodes, edges, etc. - given by the node's
   key); the two messages are printed while dumping the
   distribution; if the last argument is true, then each
   value in the distribution is also printed as a base
   2 logarithm */
void printDistribution (BST<unsigned long long int, unsigned long long int> &distr,
                        const char*, const char*, bool isIPSpace);

/* This function is identical to the previous one, except
   for the fact that it prints distributions of floating
   point values as percent values */
void printDistribution (BST<unsigned long long int, double> &distr,
                        const char*, const char*);

/* This function is used when executing a `show' command;
   see its on-line help for a more detailed explanation
   of what it can actually do */
void showCount(command_target target[], long int ccNum,
               char *currentFile, ASGraph *graph,
               ConnCompArray sccs, long int scclen,
               ConnCompArray ccs, long int cclen);

/* This function releases memory that had been previously
   allocated for the graph and its (strongly) connected
   components; it corresponds to the use of the `clear'
   command */
void clearMem(ASGraph **graph,
              ConnCompArray &sccs, long int &scclen,
              ConnCompArray &ccs, long int &cclen);
              
/* This function saves to a file a (strongly) connected
   component of index `ccNum'; it corresponds to the
   usage of the `save' command */
void saveGraph(char *fileName, char *currentFile,
               ASGraph *graph, command_target target[],
               long int ccNum, ConnCompArray sccs,
               long int scclen,
               ConnCompArray ccs, long int cclen,
               bool forceOverwrite);

/* This function reads a graph file, stores it in memory
   and computes its (strongly) connected components;
   it is used when issuing the `load' command */
void loadGraph(char *fileName, char *currentFile, ASGraph **graph,
               ConnCompArray &sccs, long int &scclen,
               ConnCompArray &ccs, long int &cclen);

/* This function is invoked when using the `echo' command;
   it either prints out a given message or enables/disables
   command echoing, by changing the value of its second
   argument */
void echoString(char*, bool &echo);

/* This function takes a token (word) as input, and tests
   how many keywords match that token (which can be a command/argument
   in its shortened form); `cmdLevel' is the index of the
   command/argument that is being processed (it is used in order to
   only allow matches for that argument), while `completeToken'
   is filled (on return) with the first matching keyword (in its
   extended version); returned value is the number of
   matching keywords */
int matchingKeywords(char *token, char *completeToken, int cmdLevel);

/* This function parses a command line and, if there are
   no errors, runs the requested command; if any error is
   found, a short explanation is printed out; returned value
   is true if issued command was an exit command; false
   otherwise */
bool executeCommand(char *cmdLine, char *currentFile, bool &echo,
                    ASGraph **graph, ConnCompArray &sccs, long int &scclen,
                    ConnCompArray &ccs, long int &cclen,
                    bool forceOverwrite);

/* The following function just starts a cycle that
   allows user to issue commands by using an on-line
   prompt */
void goInteractive(const cmdLineParameters &params);

/* This function parses command line parameters and
   returns them inside a parameter structure; if any
   syntax error occurs, returned value is false, and
   a short explanation is reported inside `error';
   otherwise, returned value is true */
bool parseArguments(int argc, char **argv,
                    cmdLineParameters &params,
                    char *error);

// This function prints out program name
void printHeader(FILE*);

// This function prints a command line usage explanation
void printHelp(FILE*, char *cmdLine);

/* This function reads a script file and runs
   its commands */
void runScript(FILE*, const cmdLineParameters &params);

#endif
